# Generated by Django 4.0.1 on 2022-01-17 13:48

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('initiator', models.CharField(max_length=250, verbose_name='Initiator')),
                ('timestamp', models.DateTimeField(auto_now_add=True, verbose_name='Date')),
                ('level', models.IntegerField(choices=[(0, 'NOTSET'), (20, 'INFO'), (30, 'WARNING'), (10, 'DEBUG'), (40, 'ERROR'), (50, 'CRITICAL')], default=0, verbose_name='Level')),
                ('event', models.TextField(default='', verbose_name='Event')),
                ('tracing', models.TextField(default='', verbose_name='Tracing')),
            ],
            options={
                'verbose_name': 'Event',
                'verbose_name_plural': 'Events',
                'db_table': 'eventlog',
                'ordering': ('-timestamp',),
            },
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class EventlogConfig(AppConfig):
    name = 'eventlog'

    def ready(self):
        self.verbose_name = _('Eventlog')



import logging
from collections import namedtuple

from django.apps import apps

# region Levels

_Level = namedtuple('_Level', 'value label color bg_color')
notset = _Level(
    logging.NOTSET,
    logging.getLevelName(logging.NOTSET),
    '#fff',
    '#3498db'
)
info = _Level(
    logging.INFO,
    logging.getLevelName(logging.INFO),
    '#fff',
    '#62cb31'
)
warning = _Level(
    logging.WARNING,
    logging.getLevelName(logging.WARNING),
    '#fff',
    '#ffb606'
)
debug = _Level(
    logging.DEBUG,
    logging.getLevelName(logging.DEBUG),
    '#fff',
    '#6a6c6f'
)
error = _Level(
    logging.ERROR,
    logging.getLevelName(logging.ERROR),
    '#fff',
    '#e74c3c'
)
critical = _Level(
    logging.CRITICAL,
    logging.getLevelName(logging.CRITICAL),
    '#fff',
    '#c0392b'
)

_Levels = namedtuple(
    '_Levels',
    'notset, info, warning, debug, error, critical'
)
Levels = _Levels(notset, info, warning, debug, error, critical)

# endregion


class EventLogHandler(logging.Handler):
    def emit(self, record):
        event_model = apps.get_model('eventlog', 'Event')
        trace = ""

        if record.exc_info:
            db_default_formatter = logging.Formatter()
            trace = db_default_formatter.formatException(record.exc_info)

        event = record.getMessage()
        event_model.objects.create(
            initiator=record.name,
            level=record.levelno,
            event=event,
            tracing=trace
        )



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


def local_scheme(version):
    return ""


setup(
    name='django-logging-eventlog',
    version=__import__('eventlog').__version__,
    setup_requires=['setuptools_scm'],
    packages=['eventlog', 'eventlog.migrations'],
    include_package_data=True,
    license='MIT License',
    description='Logger for the logging module that writes messages to the database',
    long_description=README,
    url='https://github.com/mnemchinov/django-logging-eventlog',
    author='mnemchinov',
    author_email='mnemchinov@mail.ru',
    install_requires=['django>=3.0'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Framework :: Django',
        'Environment :: Web Environment',
        'Natural Language :: Russian',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development',
    ],
)



from datetime import datetime, timedelta
from django.utils.translation import gettext_lazy as _

from django.db import models
from django.utils.safestring import mark_safe

from eventlog.services import Levels


class EventLevels(models.IntegerChoices):
    notset = Levels.notset.value, Levels.notset.label
    info = Levels.info.value, Levels.info.label
    warning = Levels.warning.value, Levels.warning.label
    debug = Levels.debug.value, Levels.debug.label
    error = Levels.error.value, Levels.error.label
    critical = Levels.critical.value, Levels.critical.label


class EventLogManager(models.Manager):
    def purge(self, days=30):
        return self.filter(
            timestamp__gt=datetime.now() - timedelta(days=days)).delete()


class Event(models.Model):

    id = models.AutoField(primary_key=True)
    initiator = models.CharField(verbose_name=_('Initiator'), max_length=250)
    timestamp = models.DateTimeField(_('Date'), auto_now_add=True)
    level = models.IntegerField(
        verbose_name=_('Level'),
        choices=EventLevels.choices,
        default=0
    )
    event = models.TextField(verbose_name=_('Event'), default="")
    tracing = models.TextField(verbose_name=_('Tracing'), default="")
    objects = EventLogManager()

    class Meta:
        ordering = ('-timestamp',)
        db_table = 'eventlog'
        verbose_name = _('Event')
        verbose_name_plural = _('Events')

    def __str__(self):
        return f'{self.get_level_display()}: {self.event[:100]}...'

    @property
    def level_label(self):
        event_log_level = [
            level for level in Levels if level[0] == self.level
        ][0]
        label = event_log_level.label
        color = event_log_level.color
        bg_color = event_log_level.bg_color
        style = f'display: inline; padding: .2em .6em .3em; font-size: 75%; ' \
                f'font-weight: bold; text-align: center; white-space: ' \
                f'nowrap; vertical-align: baseline; color: {color}; ' \
                f'background-color: {bg_color}; border-radius: .25em; '
        level_label = f'<span style="{style}">{label}</span>'

        return mark_safe(level_label)



from django.contrib import admin
from .models import Event
from django.utils.translation import gettext_lazy as _


@admin.register(Event)
class EventAdmin(admin.ModelAdmin):

    list_display = ('initiator', 'level_display', 'timestamp', 'event',)
    search_fields = ('initiator', 'event', )
    list_filter = ('level', 'timestamp', 'initiator', )

    def level_display(self, obj):
        return obj.level_label

    level_display.short_description = _('Level')

    def get_readonly_fields(self, request, obj=None):
        return [i.name for i in obj._meta.get_fields()]

    def has_add_permission(self, request):
        return False
